import 'package:logging/logging.dart';
import 'package:rook_users/rook_users.dart';
import 'package:rook_users/src/authorization/authorization_manager.dart';
import 'package:rook_users/src/data/preferences/users_preferences.dart';
import 'package:rook_users/src/data/server/dto/create_rook_user_request.dart';
import 'package:rook_users/src/data/server/users_api.dart';
import 'package:rook_users/src/domain/environment/get_api_url_from_rook_users_environment.dart';

/// Helper class to manage all users operations.
class RookUsersManager {
  final Logger _logger = Logger('RookUsersManager');

  final UsersPreferences _preferences = UsersPreferences();
  late UsersApi api;

  /// Create a new [RookUsersManager] instance.
  ///
  /// [apiUrl] - Users API url without https.
  RookUsersManager(String clientUUID, String clientPassword) {
    api = UsersApi(
      apiUrl: getApiUrlFromRookUsersEnvironment(
        RookUsersConfiguration.getEnvironment(),
        forceDevelop: false,
      ),
      clientUUID: clientUUID,
      clientPassword: clientPassword,
    );
  }

  /// Registers provided userID in server and saves it to preferences.
  ///
  /// If the user was previously registered it will return the stored preferences value.
  ///
  /// * [userID] is defined by you, so feel free to use the same ID your app uses to identify it's users.
  /// * [userType] - Were the user account will be registered.
  Future<RookUser> registerRookUser(
    String userID,
    UserType userType,
  ) async {
    AuthorizationManager().checkAuthorization();

    final user = await _preferences.getRookUser(userType);

    if (user != null && user.id == userID) {
      _logger.info('There is a user with the same ID, returning local user...');
      return user;
    }

    final newUser = RookUser(id: userID, type: userType);

    final request = CreateRookUserRequest(
      sourceOfData: userType.sourceOfData,
      userID: userID,
    );

    await api.createRookUser(request);
    await _preferences.saveRookUser(newUser);

    _logger.info('Saved user: $newUser');

    return newUser;
  }

  /// Deletes the corresponding user to the provided [UserType].
  ///
  /// This does not deletes the user from server.
  Future<bool> removeUserFromPreferences(UserType userType) {
    return _preferences.deleteRookUser(userType);
  }
}
